Multiple Definition of `XYZ()';
-------------------------------

When building a program, the last step is to link together
all of the object (.o) files necessary to make a complete
program. The linker is used to complete this step. The linker
adds system library code that connects the program
with the operating system. That system library code calls
the function named `main()` as the starting point of the 
executable program. This makes having `main()` in your program
a requirement.

A linker command looks like this:

    g++ -std=c++11 -o desired_executable_program_name object_file.o another_object_file.o ... 
    
The `-o desired_executable_program_name` part must be given together,
and tells the linker what to name the program. The rest of the line
tells the linker which object files contain the complete set of
definitions to make a complete file.

The linker makes sure that every function *call* (e.g. `fred()`) in
the object files can be matched up with a single function definition
(e.g. `void fred() { std::cout << "Fred Flintstone" << std::endl; }`)
from the object files.

If the linker doesn't find any available definitions, it will display
an error message that there are unresolved symbols, or undefined symbols,
or something to that end.

If the linker finds too many (more than 1) for a particular definition,
it will display an error message that there are multiple definitions
of the function.

The code example available here tries to build three different programs.
Each of the programs will end up with a `multiple definitions` error, 
but for different reasons.

First Cause of Multiple Definitions Error
-----------------------------------------

`program1` has a `Makefile` entry that looks like this:

    program1: program1.o fred.o barney.o
        g++ -std=c++11 -o program1 program1.o fred.o barney.o
        
`program1.cpp` contains the definition of `main()`, which calls `fred()` and `barney()`.

`fred.cpp` contains a definition of `fred()`.

`barney.cpp` contains a definition of `barney()`, but also a definition of `fred()`.

So, every function that needs a definition has been defined in a .o file.
The linker command, is told the names of each of the .o files. This is the
error message produced:

    g++ -std=c++11 -o program1 program1.o fred.o barney.o
    /usr/bin/ld: barney.o: in function `fred()':
    barney.cpp:(.text+0x36): multiple definition of `fred()'; fred.o:fred.cpp:(.text+0x0): first defined here
    collect2: error: ld returned 1 exit status

What is the cause of the error? `fred()` has been defined in `fred.cpp` and in `barney.cpp`.

What is the solution?


Second Cause of Multiple Definitions Error
------------------------------------------

`program2` has a `Makefile` entry that looks like this:

    program2: program2.o fred.o
        g++ -std=c++11 -o program2 program1.o fred.o program2.o

This is the error message produced:

    g++ -std=c++11 -o program2 program1.o fred.o program2.o
    /usr/bin/ld: program2.o: in function `main':
    program2.cpp:(.text+0x0): multiple definition of `main'; program1.o:program1.cpp:(.text+0x0): first defined here
    /usr/bin/ld: program1.o: in function `main':
    program1.cpp:(.text+0xe): undefined reference to `barney()'
    collect2: error: ld returned 1 exit status

What is the cause of the error? `main()` is in `program1.cpp` and in `program2.cpp`.
Only, that's not really a problem. Each are meant to be the starting 
point for different programs. The cause is that `program1.o` was
given to the linker when building the `program2` executable.

What is the solution?


Third Cause of Multiple Definitions Error
------------------------------------------

`program3` has a `Makefile` entry that looks like this:

    program3: program3.o fred.o
        g++ -std=c++11 -o program3 program3.o fred.o


The contents of `program3.cpp` are:

    #include "headers.h"
    #include "fred.cpp"

    int main() {
      fred();
      return 0;
    }


This is the error message produced:


    g++ -std=c++11 -o program3 program3.o fred.o
    /usr/bin/ld: fred.o: in function `fred()':
    fred.cpp:(.text+0x0): multiple definition of `fred()'; program3.o:program3.cpp:(.text+0x0): first defined here
    collect2: error: ld returned 1 exit status


What is the cause of the error? Because `fred.cpp` was included in `program3.cpp`,
`program3.o` also has a definition of `fred()`.

What is the solution?







Answers:

- For the first problem, remove one of the definitions of `fred()`,
  either from `fred.cpp` or from `barney.cpp`.
  There can be only one!
- For the second problem, remove `program1.o` from the linker command
  in the `Makefile`.
- For the third problem, remove `#include "fred.cpp"` from `program3.cpp`.
  You should *never* include a .cpp file.



