Inheritance Example
-------------------
Features of inheritance to be demonstrated:

- constructor, destructor chaining.
- public, protected, private sections of class.
- method inheritance by derived classes.
- method overriding by derived classes.
- parent class method invocation.
- polymorphic methods.


Constructor Chaining
--------------------
When an object of a child class is constructed,
the parent class part must be constructed first.
Two options exist.  1) The child class constructor
explicitly invokes the desired parent class
constructor; 2) the compiler implicitly calls
the parent class default constructor.

In the example code, look for explicit constructor
chaining when the parent class constructor is
called from a child class initialization section.

    Child::Child( list, of, parameters, to, child )
      : Parent( argument1, argument2 ), childDataMember( value ) {
    }

Implicit constructor chaining with default a constructor
can also be found. The parent constructor is not visible
in the initialization section.

    Child::Child( list, of, parameters, to, child )
      : childDataMember( value ) {
    }

The flexibility allows for many options in initializing
the data members of objects.

Destructor Chaining
-------------------
When an object of a child class is destructed,
the child class part is first destructed, then
the parent class part is destructed.  The
parent class destructor is always called
implicitly by the compiler.

    Child::~Child( ) {
      // clean up child's resources
    }

In the example code, implicit destructor chaining
occurs for all child classes.


Access Specifiers
-----------------
Access specifiers in the class declaration
determine which code has access to members
(both data and methods) declared in each
section.

public:
  Declarations in the public section of the
class are visible and accessible by all other
code that can access the class.  These are
methods and data members that the rest of
the program is intended to access directly.

private:
  Declarations in the private section of the
class are visible and accessible only to
the methods of the class.

protected:
  Declarations in the protected section of the
class are visible and accessible to the methods
of the class and to the methods of any derived
classes.

The private section is NOT accessible by
derived classes.

Usually, data members and internal methods are
protected or private, while external methods are
public.

    class Parent {
    public:
      Parent( );
      int getFred( );
      int getBarney( );
    protected:
      void yabaDabaDoo( );
      int mFred
    private:
      void BamBam( );
      int mBarney;
    };


Code external to the Parent class can construct Parent objects
with the default constructor, call getFred() or getBarney(),
but can not call yabaDabaDoo() or BamBam(), nor can it access
mFred and mBarney.

Parent class methods can all access everything, all data members
and all methods.

    class Child1: public Parent {
      // ...
    };


Code external to the Child1 class can access all public members
of the Parent class, through objects of the Child1 class.  Just
as before.

Child1 class methods can access Parent(), getFred() and getBarney().
They can also access yabaDabaDoo() and mFred.  They can not access
BamBam() or mBarney().

    class Child2: protected Parent {
      // ...
    };
    
Code external to the Child2 class can not access any members
of the Parent class, through objects of the Child2 class. The
public is not allowed to know that the inheritance occurred.

Child2 class methods can access Parent(), getFred() and getBarney().
They can also access yabaDabaDoo() and mFred.  They can not access
BamBam() or mBarney().  The child class can know that it inherited
from the parent

    class Child3: private Parent {
      // ...
    };
    
    
Code external to the Child3 class can not access any members
of the Parent class, through objects of the Child3 class. The
public is not allowed to know that the inheritance occurred.

Child3 class methods can access Parent(), getFred() and getBarney().
They can also access yabaDabaDoo() and mFred.  They can not access
BamBam() or mBarney().  The child class can know that it inherited
from the parent

    class GrandChild1: public Child1 {
      // ...
    };

Code external and internal to the GrandChild1 can access public
members of the Parent class, through objects of GrandChild1.

Code internal to GrandChild1 can access all public and protected
members of the Parent class.
    
    class GrandChild2: public Child2 {
      // ...
    };
    

Code external to the GrandChild2 can not access any members of 
the Parent class, through objects of GrandChild2. Because Child2's
inheritance was protected.

Code internal to GrandChild2 can access all public and protected
members of the Parent class.
    
    class GrandChild3: public Child3 {
      // ...
    };
    
Code external to the GrandChild3 can not access any members of 
the Parent class, through objects of GrandChild3. Because Child3's
inheritance was private.

Code internal to GrandChild3 can not access any members of the
the Parent class, because Child3's inheritance was private to
Child3.
    



Method Inheritance
------------------
Derived classes automatically inherit the methods
of the base class.


Method Overriding
-----------------
A derived class can provide its own version of
a method with the same function signature as a
base class method.  In this case, objects of the
base class continue to use the base class version,
while objects of the derived class use the derived
class version.

This is different than function overloading, where
different signatures distinguish between functions
with the same name.


Invoking a Parent Class Method
------------------------------
If a derived class wants to explicitly
call the base class method, even though
the method is overridden, this can be
accomplished with an explicit syntax.

    int Child::someMethod( ) {
      int x = Parent::someMethod();
      return x * 2;
    }


Polymorphic Methods
-------------------
It is often useful to have a collection
of objects that are not of the same type,
but need to be treated similiarly.

For example, in a drawing program, circles,
rectangles and lines are all different, but
all have a color and all need to be drawn.

This is accomplished by storing the address
of derived class objects in base class
pointers.

Virtual methods are polymorphic methods.  When
the compiler sees a call to a virtual method,
it implicitly adds code to determine at run time,
which type of object is actually pointed to.
So, even though the pointer is Parent pointer, if 
the object pointed to is a Child object, at run time,
the VTABLE is consulted to determine which version
of the method to use.  This is an example of
polymorphism.


